@@ -4,14 +4,14 @@
 ## Author:      Graciliano M. P.
 ## Modified by: Geoffrey Leach
 ## Created:     2004-01-11
-## Updated:	    2008-02-14
-## Copyright:   (c) 2004 Graciliano M. P. (c) 2007, 2008 Geoffrey Leach
+## Updated:	    2011-08-13
+## Copyright:   (c) 2004 Graciliano M. P. (c) 2007 - 2013 Geoffrey Leach
 ## Licence:     This program is free software; you can redistribute it and/or
 ##              modify it under the same terms as Perl itself
 #############################################################################
 
 package Pod::HtmlEasy;
-use 5.006002;
+use 5.006003;
 
 use strict;
 use warnings;
@@ -19,21 +19,25 @@ use warnings;
 use Pod::HtmlEasy::Parser;
 use Pod::HtmlEasy::Data
     qw( EMPTY NL NUL TRUE FALSE body css gen head headend title top toc toc_tag podon podoff );
-use File::Slurp;
 use Carp;
 use English qw{ -no_match_vars };
+use File::Slurp;
 use Readonly;
 use Regexp::Common qw{ whitespace };
 
-#use version; our $VERSION = qv("1.0.0"); # Also appears in "=head1 VERSION" in the POD below
-our $VERSION = 1.0000;
+use version;
+our $VER = qv('1.1.10');    # Also appears in "=head1 VERSION" in the POD below
+
+# Why this? CPAN (a/o 1/1/2008) does not grok qv.
+our $VERSION = '1.1.10';
 
 ########
 # VARS #
 ########
 
-Readonly::Scalar my $NUL            => NUL;
-Readonly::Scalar my $TITLE_TEXT_LOC => -2;
+Readonly::Scalar my $NUL                  => NUL;
+Readonly::Scalar my $TITLE_TEXT_LOC       => -2;
+Readonly::Scalar my $DEFAULT_INDEX_LENGTH => 60;
 
 # This keeps track of valid options
 Readonly::Hash my %OPTS => (
@@ -41,6 +45,8 @@ Readonly::Hash my %OPTS => (
     css          => 1,
     index        => 1,
     index_item   => 1,
+    index_length => 1,
+    output       => 1,
     no_css       => 1,
     no_generator => 1,
     no_index     => 1,
@@ -57,15 +63,15 @@ Readonly::Hash my %OPTS => (
 sub _organize_callbacks {
     my $this = shift;
 
-    $this->{ON_B} = \&evt_on_B;
-    $this->{ON_C} = \&evt_on_C;
-    $this->{ON_E} = \&evt_on_E;
-    $this->{ON_F} = \&evt_on_F;
-    $this->{ON_I} = \&evt_on_I;
-    $this->{ON_L} = \&evt_on_L;
-    $this->{ON_S} = \&evt_on_S;
-    $this->{ON_X} = \&evt_on_X;    # [20078]
-    $this->{ON_Z} = \&evt_on_Z;
+    $this->{ON_B} = \&evt_on_b;
+    $this->{ON_C} = \&evt_on_c;
+    $this->{ON_E} = \&evt_on_e;
+    $this->{ON_F} = \&evt_on_f;
+    $this->{ON_I} = \&evt_on_i;
+    $this->{ON_L} = \&evt_on_l;
+    $this->{ON_S} = \&evt_on_s;
+    $this->{ON_X} = \&evt_on_x;    # [20078]
+    $this->{ON_Z} = \&evt_on_z;
 
     $this->{ON_HEAD1} = \&evt_on_head1;
     $this->{ON_HEAD2} = \&evt_on_head2;
@@ -122,14 +128,25 @@ sub new {
 # POD2HTML #
 ############
 
-sub pod2html {
+sub pod2html {    ## no critic (ProhibitExcessComplexity)
     my @args = @_;
-
     my $this = shift @args;
-    my $file = shift @args;
 
-    # Assume a non-option second arg is a file name
-    my $save = exists $OPTS{ $args[0] } ? undef : shift @args;
+    # The first argument is either the input file or an option,
+    # In the latter case, input must be coming from STDIN
+    my $pod = shift @args;
+    if ( exists $OPTS{$pod} ) {
+
+        # Oops, its an arg;
+        unshift @args, $pod;
+        $pod = q{-};
+    }
+
+    # If the following assignment is to work, we must have pairs in @args
+    if ( @args & 1 ) {
+        carp q{All options must be paired with values};
+        exit 1;
+    }
     my %args = @args;
 
     # Check options for validity
@@ -139,13 +156,8 @@ sub pod2html {
         }
     }
 
-    # No /x please
-    # \r\n is said to be not portable
-    if ( defined $save && $save =~ m{\cJ\cM}ms ) {
-
-        # Is this a M$ way of saying "nothing there"?
-        $save = undef;
-    }
+    my $save;
+    if ( exists $args{output} ) { $save = $args{output}; }
 
    # Personal pecularity: I hate double negatives, and perlcritic hates unless
     my ( $do_css, $do_generator, $do_index, $do_content );
@@ -159,7 +171,10 @@ sub pod2html {
     # Pod::HtmlEasy::Parser does not implement new()
     my $parser = Pod::HtmlEasy::Parser->new();
 
-    $parser->errorsub( sub { Pod::HtmlEasy::Parser::_errors( $parser, @_ ); }
+    $parser->errorsub(
+        sub {    ## no critic (ProtectPrivateSubs)
+            Pod::HtmlEasy::Parser::_errors( $parser, @_ );
+        }
     );
 
  # Pod::Parser wiii complain about multiple blank lines in INDEX_ITEMthe input
@@ -175,22 +190,30 @@ sub pod2html {
     # Pod::Parser and $this to Pod::HtmlEasy
     $parser->{POD_HTMLEASY} = $this;
 
-    if ( exists $args{index_item} ) { $parser->{INDEX_ITEM} = 1; }
+    if ( exists $args{index_item} ) {
+        $parser->{INDEX_ITEM} = 1;
+        $parser->{INDEX_LENGTH}
+            = exists $args{index_length}
+            ? $args{index_length}
+            : $DEFAULT_INDEX_LENGTH;
+    }
 
     # This is where we accumulate the results of Pod::Parser
     my @output;
     $parser->{POD_HTMLEASY}->{HTML} = \@output;
 
     my $title = $args{title};
-    if ( ref $file eq q{GLOB} ) {    # $file is an open file handle
+    if ( ref $pod eq q{GLOB} ) {    # $pod is an open file handle
         if ( not defined $title ) { $title = q{<DATA>}; }
     }
     else {
-        if ( !-e $file ) {
-            carp qq{No file $file};
-            return;
+        if ( ( !-e $pod ) && ( $pod ne q{-} ) ) {
+            carp qq{No file $pod};
+            exit 1;
+        }
+        if ( not defined $title ) {
+            $title = defined $save ? $save : $pod eq q{-} ? q{STDIN} : $pod;
         }
-        if ( not defined $title ) { $title = defined $save ? $save : $file; }
     }
 
     # Build the header to the HTML file
@@ -199,7 +222,7 @@ sub pod2html {
         push @html, head();
 
         if ( defined $do_generator ) {
-            push @html, gen( $VERSION, $Pod::Parser::VERSION );
+            push @html, gen( $VER, $Pod::Parser::VERSION );
         }
 
         push @html, title($title);
@@ -232,7 +255,7 @@ sub pod2html {
     delete $this->{VERSION};
     $this->{INFO_COUNT} = 0;
 
-    $parser->parse_from_file($file);
+    $parser->parse_from_file($pod);
 
     # If there's a head1 NAME, we've picked this up during processing
     # BUT, let the caller force override of NAME content
@@ -256,10 +279,13 @@ sub pod2html {
     @html = map { $_ . NL } @html;
 
     if ( defined $save ) {
-        open my $out, q{>}, $save or carp qq{Unable to open $save - $!};
+        open my $out, q{>}, $save or carp qq{Unable to open $save - $ERRNO};
         print {$out} @html or carp qq{Could not write to $out};
         close $out or carp qq{Could not close $out};
     }
+    else {
+        if ( $pod eq q{-} ) { print @html or carp q{Could not print}; }
+    }
 
     return wantarray ? @html : join EMPTY, @html;
 }
@@ -314,7 +340,7 @@ sub _do_index {
             }
 
             # Strip http to conform to =item
-            $txt =~ s{\Ahttps?://}{}gmx;
+            $txt =~ s{\Ahttps?://}{}gmsx;
             $tag = toc_tag($txt);
         }
 
@@ -421,19 +447,25 @@ sub evt_on_head4 {
 
 sub evt_on_begin {
     my ( $this, $txt ) = @_;
-    $this->{IN_BEGIN} = 1;
+
+    # We don't do any processing for =begin/=end other than ignore
+    # However, without a command, the construct is illegal
+    # Embedded =head, etc are also illegal, but we don't check
+    if ( length $txt == 0 ) { $this->{IN_BEGIN} = 1; }
     return EMPTY;
 }
 
 sub evt_on_end {
     my ( $this, $txt ) = @_;
+
+    # Ignore any commands
     delete $this->{IN_BEGIN};
     return EMPTY;
 }
 
 # See perlpodsec for details on interpreting the items
-sub evt_on_L {
-    my ( $htis, $text, $inferred, $name, $section, $type ) = @_;
+sub evt_on_l {    ## no critic (ProhibitManyArgs)
+    my ( $this, $text, $inferred, $name, $section, $type ) = @_;
 
     if ( $type eq q{pod} ) {
         $section = defined $section ? qq{#$section} : EMPTY;    # [6062]
@@ -441,19 +473,17 @@ sub evt_on_L {
         $inferred =~ s{\A(.)}{$1$NUL}smx;
         my $toc_tag = toc_tag($section);
 
-        # Possible bug in Pod::Parser
-        #if ( defined $name && not length $name ) { $name = undef; }
-        return
-            defined $name
-            ? qq{<i><a href='h${NUL}ttp://search.cpan.org/perldoc?}
-            . qq{$name$section'>$inferred</a></i>}
-            : qq{<i><a href='$toc_tag'>$inferred</a></i>}
-            ;    # Internal reference
+        if ( defined $name ) {
+            return qq{<i><a href='h${NUL}ttp://search.cpan.org/perldoc?}
+                . qq{$name$section'>$inferred</a></i>};
+        }
+        return qq{<i><a href='$toc_tag'>$inferred</a></i>};
     }
+
     if ( $type eq q{man} ) {
 
  # $name probably looks like "foo(1)", and the () are interpreted as metachars
-        if ( $inferred !~ m{\Q$name\E}mx ) { $inferred .= qq{ in $name}; }
+        if ( $inferred !~ m{\Q$name\E}msx ) { $inferred .= qq{ in $name}; }
         return qq{<i>$inferred</i>};
     }
     if ( $type eq q{url} ) {
@@ -466,22 +496,22 @@ sub evt_on_L {
     return $inferred;
 }
 
-sub evt_on_B {
+sub evt_on_b {
     my ( $this, $txt ) = @_;
     return qq{<b>$txt</b>};
 }
 
-sub evt_on_I {
+sub evt_on_i {
     my ( $this, $txt ) = @_;
     return qq{<i>$txt</i>};
 }
 
-sub evt_on_C {
+sub evt_on_c {
     my ( $this, $txt ) = @_;
-    return qq{<font face='Courier New'>$txt</font>};
+    return qq{<code>$txt</code>};
 }
 
-sub evt_on_E {
+sub evt_on_e {
     my ( $this, $txt ) = @_;
 
     $txt =~ s{^&}{}smx;
@@ -490,12 +520,12 @@ sub evt_on_E {
     return qq{&$txt;};
 }
 
-sub evt_on_F {
+sub evt_on_f {
     my ( $this, $txt ) = @_;
     return qq{<b><i>$txt</i></b>};
 }
 
-sub evt_on_S {
+sub evt_on_s {
     my ( $this, $txt ) = @_;
 
     # Eliminate newlines; dos files use \r\n
@@ -504,9 +534,9 @@ sub evt_on_S {
     return $txt;
 }
 
-sub evt_on_X { return EMPTY; }    # [20078]
+sub evt_on_x { return EMPTY; }    # [20078]
 
-sub evt_on_Z { return EMPTY; }
+sub evt_on_z { return EMPTY; }
 
 sub evt_on_verbatim {
     my ( $this, $txt ) = @_;
@@ -548,7 +578,7 @@ sub evt_on_over {
 sub evt_on_item {
     my ( $this, $txt ) = @_;
 
-    if ( $txt eq q{*} ) {
+    if ( ( length($txt) == 1 ) && ( $txt !~ m{\d}msx ) ) {
 
         # Use the content for the tag
         $this->{IN_ITEM} = 1;
@@ -647,13 +677,53 @@ __END__
 
 =pod
 
+=begin stopwords
+
+PODs
+CPAN
+html
+FILEHANDLE
+STDIN
+STDOUT
+pre
+css
+HREF
+intex
+DOCTYPE
+outptut
+parserwarn
+backend
+Regretably
+uArr
+CSS
+undef
+avalable
+nonblank
+automagically
+encodings
+URIs
+http
+https
+mailto
+Firefox
+mis
+HtmlEasy
+Graciliano
+Tubert
+Brohman
+Nobuaki
+Whitcomb
+Wieselquist
+
+=end stopwords
+
 =head1 NAME
 
 Pod::HtmlEasy - Generate personalized HTML from PODs.
 
 =head1 VERSION
 
-This documentation refers to Pod::HtmlEasy version 1.0.0.
+This documentation refers to Pod::HtmlEasy version 1.1.10.
 
 =head1 DESCRIPTION
 
@@ -679,13 +749,31 @@ list, according to context.
 The POD file (file path) or FILEHANDLE (GLOB, opened).
 The special file handle, DATA is, of course, supported.
 
-=item HTML_FILE I<(optional)>
+If the POD file is "-", or is omitted altogether, input from STDIN is expected, and HTML is written
+to STDOUT, unless an output file name has been given.
 
-The output HTML file path. If you don't need this, just leave it out.
-In other words, no "undef" (or something similar) is required.
+This command shows how to convert POD to HTML on the command line:
+
+ C<perl -MPod::HtmlEasy -e'Pod::HtmlEasy->new->pod2html(title,"test.html")' < test.pod > test.html>
+
+or
+
+ C<perl -MPod::HtmlEasy -e'Pod::HtmlEasy->new->pod2html("-",title,"test.html")' < test.pod > test.html>
+
+The "title,test" shows how to set parameters to pod2html in this context.
+Note that there is no "-" preceding the title; if you use one, Perl will complain about
+an odd number of values in an hash assignment.
+
+=item HTML_FILE
+
+The default is to use the POD_FILE parameter, replacing the extension with "html"
+in the current directory. If you want to name the output file differently, use the I<-output>
+option. B<Note that this is an incompatible change from pre-1.0 versions. Sorry 'bout that.>
 
 =item %OPTIONS I<(optional)>
 
+Note that B<all> options have values. Omit the value and you'll get dumped.
+
 =over
 
 =item body
@@ -736,11 +824,18 @@ the required HTML glue.
 
 =item index_item
 
-If set (1), =items will be added in the index.
+If set (1), =items will be added in the index. =item *, followed by a paragraph can produce some
+strange indexes. See index_length.
 
 If the =item line ("foo" in =item foo) is an URL (https?://...), whether or not its enclosed
 in LZ<>E<lt>E<gt>, the http?// is stripped, and a HTML link is created.
 
+=item index_length
+
+If set (some value) I<and> index_item is set, then the intex line will be restricted to the
+first space following this
+number of characters, followed by "..." if appropriate. Default 60 characters.
+
 =item no_css
 
 If set do not use css.
@@ -758,6 +853,10 @@ If set, the meta GENERATOR tag won't be added.
 If set generate only the HTML content. This I<implies> no_generator and no_css,
 produces no <body> or <title>, and no DOCTYPE as well, so its really not very good HTML.
 
+=item output
+
+The file (and path, if desired) to be used to write the outptut HTML.
+
 =item parserwarn
 
 The backend we use is Pod::Parser. This module generates warnings when it detects
@@ -1003,6 +1102,10 @@ I<If you want to do your own CSS, use this as base.>
 
 =over
 
+=item All options must be paired with values
+
+Your argument list (excluding the .pod file if specified) has an odd number of items.
+
 =item option I<key> is not supported
 
 You've used (mis-spelled?) an unrecognized option.
@@ -1017,7 +1120,7 @@ The various pm_ functions are referenced through the module.
 
 The maintainer would appreciate hearing about
 any messages I<other> than those that result from
-the C<use warnings> specified for each module. .
+the C<use warnings> specified for each module.
 
 HtmlEasy uses Pod::Parser, which may produce error messages concerning malformed
 HTML.
@@ -1026,7 +1129,7 @@ HTML.
 
 =head1 SEE ALSO
 
-L<Pod::Parser> L<perlpod>.
+L<Pod::Parser> L<perlpod>, L<perlpodspec>.
 
 =head1 CONFIGURATION AND ENVIRONMENT
 
@@ -1034,7 +1137,9 @@ Neither is relevant.
 
 =head1 INCOMPATIBILITIES
 
-None are known.
+As of version 1.1, the use of an optional output file as the second parameter,
+has been replaced with an explicit
+I<output> option.
 
 =head1 BUGS AND LIMITATIONS
 
@@ -1046,12 +1151,21 @@ Graciliano M. P. <gm@virtuasites.com.br>
 
 I will appreciate any type of feedback (include your opinions and/or suggestions). ;-P
 
+=head1 THANKS
+
 Thanks to Ivan Tubert-Brohman <itub@cpan.org> that suggested to add the basic_entities
 and common_entities options and for tests. [These options have been removed. As "modern"
 browsers don't need all that encoding. See L<CHARACTER SET> above.]. 
 
 Thanks to ITO Nobuaki for the patches for [31784].
 
+Thanks to David Whitcomb for pointing out an error in HTML generation.
+
+Thanks to William Wieselquist for [58274], in which he pointed out an error in the
+parsing of dotted user names in mail address syntax.
+
+Thanks to Zefram for providing patches for using native switch if available. [82400]
+
 =head1 MAINTENANCE
  
 Updates for version 0.0803 and subsequent by Geoffrey Leach <gleach@cpan.org>
@@ -1059,7 +1173,7 @@ Updates for version 0.0803 and subsequent by Geoffrey Leach <gleach@cpan.org>
 =head1 LICENSE AND COPYRIGHT
 
  Copyright 2004-2006 by M. P. Graciliano
- Copyright 2007-2008 by Geoffrey Leach
+ Copyright 2007-2013 by Geoffrey Leach
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
