@@ -4,14 +4,14 @@
 ## Author:      Graciliano M. P.
 ## Modified by: Geoffrey Leach
 ## Created:     11/01/2004
-## Updated:	    2008-02-14
-## Copyright:   (c) 2004 Graciliano M. P. (c) 2007, 2008 Geoffrey Leach
+## Updated:	    2010-06-13
+## Copyright:   (c) 2004 Graciliano M. P. (c) 2007 - 2013 Geoffrey Leach
 ## Licence:     This program is free software; you can redistribute it and/or
 ##              modify it under the same terms as Perl itself
 #############################################################################
 
 package Pod::HtmlEasy::Parser;
-use 5.006002;
+use 5.006003;
 
 use base qw{ Pod::Parser };
 use Pod::Parser;
@@ -24,12 +24,22 @@ use English qw{ -no_match_vars };
 use Regexp::Common qw{ whitespace number URI };
 use Regexp::Common::URI::RFC2396 qw { $escaped };
 use Pod::Escapes qw{ e2char };
-use Switch qw{ Perl6 };
+
+# Provided for RT 82400. Use native switch if available.
+BEGIN {
+        if ($PERL_VERSION >= 5.012) {
+                require feature;
+                "feature"->import(qw(switch));
+        } else {
+                require Switch;
+                "Switch"->import(qw(Perl6));
+        }
+}
 
 use strict;
 use warnings;
 
-use version; our $VERSION = qv("1.0.0");
+use version; our $VERSION = qv('1.1.10');
 
 ########
 # VARS #
@@ -37,28 +47,32 @@ use version; our $VERSION = qv("1.0.0");
 
 Readonly::Scalar my $NUL => NUL;
 
+# RT 58274 [\w-]+ => [\w\.-]
+# Commented patterns temp test
 Readonly::Scalar my $MAIL_RE => qr{
          (         # grab all of this
-         [\w-]+    # some word chars with '-' included   foo
+         [\w\.-]+  # some word chars with '-' and '.'included  foo
          \0?       # possible NUL escape
-         \@        # literal '@'                         @
-         [\w\\-]+  # another word                        bar
+         \@        # literal '@'                               @
+         [\w\.-]+  # another word                              bar
          (?:       # non-grabbing pattern
-          \.       # literal '.'                        .
-          [\w\-\.]+# that word stuff                    stuff
-          \.       # another literal '.'                .
-          [\w\-]+  # another word                       and
+#         \.       # literal '.'                               .
+          [\w\.-]+ # that word stuff                           stuff
+#         \.       # another literal '.'                       .
+          [\w\.-]+ # another word                              and
           |        # or
-          \.       # literal '.'                        .   
-          [\w\-]+  # word                               nonsense
+#         \.       # literal '.'                               .   
+          [\w\.-]+ # word                                      nonsense
           |        # or empty?
-         )        # end of non-grab
-         )        # end of grab
+         )         # end of non-grab
+         )         # end of grab
         }smx;    # [6062]
 
+
+
 # Treatment of embedded HTML-significant characters and embedded URIs.
 
-# There are some characters (%html_entities below) which may in some
+# There are some characters (%HTML_ENTITIES below) which may in some
 # circumstances be interpreted by a browser, and you probably don't want that
 # Consequently, they are replaced by names defined by the W3C UNICODE spec,
 # http://www.w3.org/TR/MathML2/bycodes.html, bracketed by '&' and ';'
@@ -86,15 +100,15 @@ Readonly::Scalar my $MAIL_RE => qr{
 # the generated URI, but after the first character. These NULs are removed
 # by _remove _nul_escapes()
 
-Readonly::Hash my %html_entities => (
+Readonly::Hash my %HTML_ENTITIES => (
     q{&} => q{amp},
     q{>} => q{gt},
     q{<} => q{lt},
     q{"} => q{quot},
 );
 
-my $HTML_ENTITIES_RE = join q{|}, keys %html_entities;
-$HTML_ENTITIES_RE = qr{$HTML_ENTITIES_RE}mx;
+my $HTML_ENTITIES_RE = join q{|}, keys %HTML_ENTITIES;
+$HTML_ENTITIES_RE = qr{$HTML_ENTITIES_RE}msx;
 
 #################
 # _NUL_ESCAPE   #
@@ -125,13 +139,15 @@ sub _remove_nul_escapes {
 ####################
 
 sub _encode_entities {
-    my ( $parser, $txt_ref ) = @_;
+    my $txt_ref = shift;
 
-    foreach my $chr ( keys %html_entities ) {
+    if ( !( defined $txt_ref && length ${$txt_ref} ) ) { return; }
+
+    foreach my $chr ( keys %HTML_ENTITIES ) {
 
         # $chr gets a lookbehind to avoid converting flagged from E<...>
         my $re = qq{(?<!$NUL)$chr};
-        ${$txt_ref} =~ s{$re}{$NUL&$html_entities{$chr};}gsmx;
+        ${$txt_ref} =~ s{$re}{$NUL&$HTML_ENTITIES{$chr};}gsmx;
     }
 
     return;
@@ -141,14 +157,14 @@ sub _encode_entities {
 # _ADD_URI_HREF #
 #################
 
-# process embedded URIs that are not noted in l<...> bracketing
+# process embedded URIs that are not noted in L<...> bracketing
 # Note that the HTML-significant characters are escaped;
 # The escapes are removed by _encode_entities
 # Note that there's no presumption that there's a URI in the
 # text, so not matching is _not_ and error.
 
 sub _add_uri_href {
-    my ( $parser, $txt_ref ) = @_;
+    my ($txt_ref) = @_;
 
     if ( ${$txt_ref} =~ m{https?:}smx ) {
 
@@ -157,16 +173,22 @@ sub _add_uri_href {
 # per the RFC. However, the Spamassassin folks use it in the host.
 # $escaped is defined by Regexp::Common::URI::RFC2396, and matches %xx
 # This is done first because if needed, the host part won't be parsed correctly
-        while ( ${$txt_ref} =~ m{($escaped)}mx ) {
+        while ( ${$txt_ref} =~ m{($escaped)}msx ) {
             my $esc = $1;
             my $new = $1;
-            $new =~ s{%}{0x}mx;
+            $new =~ s{%}{0x}msx;
             $new = e2char($new);
-            ${$txt_ref} =~ s{$esc}{$new}gmx;
+            ${$txt_ref} =~ s{$esc}{$new}gmsx;
         }
 
-        # target='_blank' causes load to a new window or tab
-        # See HTML 4.01 spec, section 6.16 Frame target names
+   # target='_blank' causes load to a new window or tab
+   # See HTML 4.01 spec, section 6.16 Frame target names
+   # Doing this because URI RE grabs non-word trailing characters
+   #       ${$txt_ref} =~ m{$RE{URI}{HTTP}{-keep}{-scheme=>'https?'}}mx;
+   #       my $uri = $1;
+   #       my $host = $3;
+   #       $uri =~ s{[^/\w]+\z}{}mx;
+   #       ${$txt_ref} =~ s{$uri}{<a href='$uri' target='_blank'>$host</a>}mx;
         ${$txt_ref}
             =~ s{$RE{URI}{HTTP}{-keep}{-scheme=>'https?'}}{<a href='$1' target='_blank'>$3</a>}gsmx;
 
@@ -216,63 +238,67 @@ sub command {
     $expansion =~ s{$RE{ws}{crop}}{}gsmx;    # delete surrounding whitespace
 
     # Encoding puts in a NUL; we're finished with the text, so remove them
-    _encode_entities( $parser, \$expansion );
+    _encode_entities( \$expansion );
     _remove_nul_escapes( \$expansion );
 
     my $html;
     given ($command) {
-        when q{head1} {
+        when (q{head1}) {
             _add_index( $parser, $expansion, $LEVEL1 );
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_HEAD1}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when q{head2} {
+        when (q{head2}) {
             _add_index( $parser, $expansion, $LEVEL2 );
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_HEAD2}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when q{head3} {
+        when (q{head3}) {
             _add_index( $parser, $expansion, $LEVEL3 );
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_HEAD3}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when q{head4} {
+        when (q{head4}) {
             _add_index( $parser, $expansion, $LEVEL4 );
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_HEAD4}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when q{begin} {
+        when (q{begin}) {
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_BEGIN}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when q{end} {
+        when (q{end}) {
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_END}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when q{over} {
+        when (q{over}) {
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_OVER}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when q{item} {
+        when (q{item}) {
 
             # Items that begin with '* ' are ugly. Is it there for pod2man?
             # Which is not the same as _only_ '*'
-            $expansion =~ s{\A\*\s+}{}mx;
+            $expansion =~ s{\A\*\s+}{}msx;
 
             if ( $parser->{INDEX_ITEM} ) {
                 _add_index( $parser, $expansion, $LEVELL );
             }
 
             # This is for the folks who use =item to list URLs
-            _add_uri_href( $parser, \$expansion );
+            if ( $expansion !~ m{<a\shref=}smx ) {
+
+                # The URI's not already encoded (L<...> is already processed)
+                _add_uri_href( \$expansion );
+            }
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_ITEM}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when q{back} {
+        when (q{back}) {
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_BACK}( $parser->{POD_HTMLEASY}, $expansion );
         }
-        when q{for} {
+        when (q{for}) {
             $html = $parser->{POD_HTMLEASY}
                 ->{ON_FOR}( $parser->{POD_HTMLEASY}, $expansion );
         }
@@ -283,7 +309,7 @@ sub command {
                     ->{qq{ON_\U$command\E}}( $parser->{POD_HTMLEASY},
                     $expansion );
             }
-            elsif ( $command !~ /^(?:pod|cut)$/imx ) {
+            elsif ( $command !~ /^(?:pod|cut)$/imsx ) {
                 $html = qq{<pre>=$command $expansion</pre>};
             }
             else { $html = EMPTY; }
@@ -299,7 +325,7 @@ sub command {
 
 ############
 # VERBATIM #
-############$parser->{POD_HTMLEASY}
+############
 
 # Overrides verbatim() provided by base class in Pod::Parser
 sub verbatim {
@@ -318,18 +344,25 @@ sub _verbatim {
     my $expansion = $parser->{POD_HTMLEASY}->{VERBATIM_BUFFER};
     $parser->{POD_HTMLEASY}->{VERBATIM_BUFFER} = EMPTY;
 
-    _encode_entities( $parser, \$expansion );
+    _encode_entities( \$expansion );
+
+    # If we had "=item *", we should now be looking at the text that will
+    # appear as the item. The "*" was passed over initially, so we need
+    # the text to index. Save the flag as ON_VERBATIM deletes IN_ITEM
+
+    my $add_index = $parser->{INDEX_ITEM} && $parser->{POD_HTMLEASY}{IN_ITEM};
 
     my $html = $parser->{POD_HTMLEASY}
         ->{ON_VERBATIM}( $parser->{POD_HTMLEASY}, $expansion );
 
     # Now look for any embedded URIs
-    _add_uri_href( $parser, \$html );
+    _add_uri_href( \$html );
 
     # And remove any NUL escapes
     _remove_nul_escapes( \$html );
 
     if ( $html ne EMPTY ) {
+        if ($add_index) { _add_index( $parser, $expansion, $LEVELL ); }
         push @{ $parser->{POD_HTMLEASY}->{HTML} }, $html;
     }    # [6062]
 
@@ -355,18 +388,25 @@ sub textblock {
     $expansion =~ s{\s+$}{}gsmx;
 
     # Encode HTML-specific characters before adding any HTML (eg <p>)
-    _encode_entities( $parser, \$expansion );
+    _encode_entities( \$expansion );
+
+    # If we had "=item *", we should now be looking at the text that will
+    # appear as the item. The "*" was passed over initially, so we need
+    # the text to index. Save the flag as ON_TEXTBLOCK deletes IN_ITEM
+
+    my $add_index = $parser->{INDEX_ITEM} && $parser->{POD_HTMLEASY}{IN_ITEM};
 
     my $html = $parser->{POD_HTMLEASY}
         ->{ON_TEXTBLOCK}( $parser->{POD_HTMLEASY}, $expansion );
 
     # Now look for any embedded URIs
-    _add_uri_href( $parser, \$html );
+    _add_uri_href( \$html );
 
     # And remove any NUL escapes
     _remove_nul_escapes( \$html );
 
     if ( $html ne EMPTY ) {
+        if ($add_index) { _add_index( $parser, $expansion, $LEVELL ); }
         push @{ $parser->{POD_HTMLEASY}->{HTML} }, $html;
     }
 
@@ -383,44 +423,53 @@ sub interior_sequence {
 
     my $ret;
 
+    # Encode HTML-specific characters before adding any HTML (eg <p>)
+    if ( $seq_command ne q{L} ) {
+        _encode_entities( \$seq_argument );
+    }
+
     given ($seq_command) {
-        when q{B} {
+        when (q{B}) {
             $ret = $parser->{POD_HTMLEASY}
                 ->{ON_B}( $parser->{POD_HTMLEASY}, $seq_argument );
         }
-        when q{C} {
+        when (q{C}) {
             $ret = $parser->{POD_HTMLEASY}
                 ->{ON_C}( $parser->{POD_HTMLEASY}, $seq_argument );
         }
-        when q{E} {
+        when (q{E}) {
             $ret = $parser->{POD_HTMLEASY}
                 ->{ON_E}( $parser->{POD_HTMLEASY}, $seq_argument );
         }
-        when q{F} {
+        when (q{F}) {
             $ret = $parser->{POD_HTMLEASY}
                 ->{ON_F}( $parser->{POD_HTMLEASY}, $seq_argument );
         }
-        when q{I} {
+        when (q{I}) {
             $ret = $parser->{POD_HTMLEASY}
                 ->{ON_I}( $parser->{POD_HTMLEASY}, $seq_argument );
         }
-        when q{L} {
+        when (q{L}) {
 
             # L<> causes problems, but not with parselink.
             if ( $seq_argument eq EMPTY ) {
                 _errors( $parser, q{Empty L<>} );
                 return EMPTY;
             }
-            $ret = $parser->{POD_HTMLEASY}->{ON_L}(
-                $parser->{POD_HTMLEASY},
-                Pod::ParseLink::parselink($seq_argument)
-            );
+            my @parsed = Pod::ParseLink::parselink($seq_argument);
+            foreach (@parsed) {
+                if ( defined $_ ) { _encode_entities( \$_ ); }
+            }
+
+            # Encoding handled in ON_L()
+            $ret = $parser->{POD_HTMLEASY}
+                ->{ON_L}( $parser->{POD_HTMLEASY}, @parsed );
         }
-        when q{S} {
+        when (q{S}) {
             $ret = $parser->{POD_HTMLEASY}
                 ->{ON_S}( $parser->{POD_HTMLEASY}, $seq_argument );
         }
-        when q{Z} {
+        when (q{Z}) {
             $ret = $parser->{POD_HTMLEASY}
                 ->{ON_Z}( $parser->{POD_HTMLEASY}, $seq_argument );
         }
@@ -507,6 +556,20 @@ sub _add_index {
     # Don't index star items
     if ( $txt eq q{*} ) { return; }
 
+    if ( exists $parser->{INDEX_ITEM} ) {
+        my $max_len = $parser->{INDEX_LENGTH};
+        if ( length $txt > $max_len ) {
+            while ( substr( $txt, $max_len, 1 ) ne q{ } ) {
+                $max_len++;
+                last if $max_len >= length $txt;
+            }
+            if ( $max_len < length $txt ) {
+                $txt = substr( $txt, 0, $max_len ) . "...";
+            }
+        }
+    }
+
+    _remove_nul_escapes( \$txt );
     push @{ $parser->{POD_HTMLEASY}->{INDEX} }, [ $level, $txt ];
 
     return;
