@@ -1,4 +1,3 @@
-#
 #===============================================================================
 #
 #         FILE:  Run.pm
@@ -9,9 +8,9 @@
 #         BUGS:  ---
 #        NOTES:  ---
 #       AUTHOR:  Geoffrey Leach, <geoff@hughes.net>
-#      VERSION:  1.0
+#      VERSION:  1.1.9
 #      CREATED:  12/20/07 13:29:02 PST
-#     REVISION:  ---
+#    COPYRIGHT:  (c) 2008-2010 Geoffrey Leach
 #===============================================================================
 
 package Run;
@@ -24,8 +23,9 @@ use Carp;
 use English qw{ -no_match_vars };
 use File::Slurp;
 use Readonly;
+use IPC::Run qw( start pump finish );
 use Test::More qw(no_plan);
-use version; our $VERSION = qv('1.0'); # Also appears in "=head1 VERSION" in the POD below
+use version; our $VERSION = qv('1.1.9');
 
 BEGIN {
     use_ok(q{Pod::HtmlEasy});
@@ -33,11 +33,10 @@ BEGIN {
         qw( EMPTY NL body css gen head headend title toc top podon podoff ) );
 }
 
-use Exporter qw( import );
-our @EXPORT_OK = qw( run html_file );
+use Exporter::Easy ( OK => [qw( run html_file )], );
 
-my $pod_file  = q{./test.pod};
-my $html_file = q{./test.html};
+my $pod_file  = q{test.pod};
+my $html_file = q{test.html};
 my $htmleasy  = Pod::HtmlEasy->new;
 ok( defined $htmleasy, q{New HtmlEasy} );
 
@@ -51,7 +50,7 @@ my %default_opts = (
     no_generator => 1,
 );
 
-sub run {
+sub run {    ## no critic (ProhibitExcessComplexity)
     my ( $desc, $pod, $expect, $inx, $opts ) = @_;
 
     $test_no++;
@@ -80,12 +79,46 @@ sub run {
         delete $opts->{htmleasy};
     }
     my @html;
-    if ( exists $opts->{outfile} ) {
+    if ( exists $opts->{stdio} ) {
+
+        # Generate code to pipe @pod to pod2html and retrieve outptut
+        # Avoid complaint option stdio not suported
+        delete $opts->{stdio};
+
+        my ( $in, $out, $err );
+
+        # Execute this
+        my @cmd = ( $EXECUTABLE_NAME, qw{-Ilib -MPod::HtmlEasy -e} );
+
+        # Note: no "'"!
+        # To test the "-" file convention, add '"-",' after the left paren
+        my $cmd = q{Pod::HtmlEasy->new->pod2html(};
+
+        # Stringify options, add to -e command
+        foreach my $k ( keys %{$opts} ) {
+            $cmd .= $k . q{,} . $opts->{$k} . q{,};
+        }
+        $cmd .= q{)};
+
+        # Complete the command
+        push @cmd, $cmd;
+        my $harness = start \@cmd, \$in, \$out, \$err;
+        foreach my $p (@pod) {
+            $in .= $p;
+            $harness->pump;
+        }
+        $harness->finish;
+        ## no critic (RequireExtendedFormatting RequireLineBoundaryMatching)
+        @html = map {qq{$_\n}} split m{\n}, $out;
+        carp $err if $err;
+    }
+    elsif ( exists $opts->{outfile} ) {
 
         # Outfile is for this;
         my $outfile = $opts->{outfile};
         delete $opts->{outfile};
-        @html = $htmleasy->pod2html( $pod_file, $outfile, %{$opts} );
+        @html
+            = $htmleasy->pod2html( $pod_file, q{output}, $outfile, %{$opts} );
     }
     else {
         @html = $htmleasy->pod2html( $pod_file, %{$opts} );
@@ -96,7 +129,7 @@ sub run {
             @expect = head();
             if ( not exists $opts->{no_generator} ) {
                 push @expect,
-                    gen( $Pod::HtmlEasy::VERSION, $Pod::Parser::VERSION );
+                    gen( $Pod::HtmlEasy::VER, $Pod::Parser::VERSION );
             }
             push @expect, title( $opts->{title} );
             if ( exists $opts->{css} ) {
